// Import all DSS packages for the Debug Server
import com.ti.ccstudio.scripting.environment.*;
import com.ti.debug.engine.scripting.*;

import java.util.*;
import java.lang.*;

public class CommandServer extends Thread
{
	private TcpServer tcpServer;
	private HashMap<String, Commands> commandMap;
	
	private ScriptingEnvironment env;
	private DebugServer debugServer;
	private DebugSession debugSession;

	private boolean close;
	
	private enum Commands
	{
		OPEN,
		CONNECT,
		DISCONNECT,
		RESET,
		LOAD,
		RUN,
		RUN_ASYNCH,
		HALT,
		IS_HALTED,
		IS_CONNECTED,
		RESTART,
		WAIT_HALT,
		GET_SYMBOL,
		READ,
		WRITE,
		SET_BREAKPOINT,
		REMOVE_BREAKPOINT,
		CLOSE,
		EVALUATE,
		MEMORY_MAP,
		SINGLE_READ,
		READ_REGISTER,
		WRITE_REGISTER,
		INVALID
	}
	
	public CommandServer()
	{
		close = false;

		tcpServer = new TcpServer();
		
		// Create our scripting environment object - which is the main entry point into any script and
        // the factory for creating other Scriptable Servers and Sessions
        env = ScriptingEnvironment.instance();
		
		commandMap = new HashMap<String, Commands>();
		commandMap.put("open", Commands.OPEN);
		commandMap.put("connect", Commands.CONNECT);
		commandMap.put("disconnect", Commands.DISCONNECT);
		commandMap.put("reset", Commands.RESET);
		commandMap.put("load", Commands.LOAD);
		commandMap.put("run", Commands.RUN);
		commandMap.put("run_asynch", Commands.RUN_ASYNCH);
		commandMap.put("halt", Commands.HALT);
		commandMap.put("is_halt", Commands.IS_HALTED);
		commandMap.put("is_connected", Commands.IS_CONNECTED);
		commandMap.put("restart", Commands.RESTART);
		commandMap.put("wait_halt", Commands.WAIT_HALT);
		commandMap.put("get_symbol", Commands.GET_SYMBOL);
		commandMap.put("read", Commands.READ);
		commandMap.put("write", Commands.WRITE);
		commandMap.put("set_breakpoint", Commands.SET_BREAKPOINT);
		commandMap.put("remove_breakpoint", Commands.REMOVE_BREAKPOINT);
		commandMap.put("close", Commands.CLOSE);
		commandMap.put("evaluate", Commands.EVALUATE);
		commandMap.put("memory_map", Commands.MEMORY_MAP);
		commandMap.put("single_read", Commands.SINGLE_READ);
		commandMap.put("read_register", Commands.READ_REGISTER);
		commandMap.put("write_register", Commands.WRITE_REGISTER);
	}
	
	public void run()
	{
		while(true)
		{
			String command = tcpServer.getCommand();
			System.out.println("received command: " + command);
			String response = executeCommand(command);
			tcpServer.sendResponse(response);
			System.out.println("sent response: " + response);

			if(close){close();}
		}
	}
	
	private String executeCommand(String message)
	{
		message = message.trim();
		
		String[] temp = message.split("\"");
		String[] params = new String[temp.length / 2];
		
		for(int iter = 0; iter < params.length; iter++)
		{
			params[iter] = temp[(iter * 2) + 1];
		}		
		
		String command = params[0];
		command = command.trim().toLowerCase();	
	
		Commands resolvedCommand = commandMap.get(command);
		if(resolvedCommand==null){resolvedCommand = Commands.INVALID;}
		
		try 
		{
			System.out.println("Executing command: " + resolvedCommand);
			switch(resolvedCommand)
			{		
					
				case OPEN:
					//The default timeout is 'infinite'.  Set ours to something shorter, 50s
					env.setScriptTimeout(60000);
					//Create a log file to log script execution
					env.traceBegin("dssScriptLog.xml", "DefaultStylesheet.xsl");
					//Make the log and console file really verbose
					env.traceSetConsoleLevel(TraceLevel.INFO);
					env.traceSetFileLevel(TraceLevel.ALL);		
					//Open a debug session					
					debugServer = (DebugServer) env.getServer("DebugServer.1");					
					debugServer.setConfig(params[1]); 
					debugSession = debugServer.openSession(".*/C28.*"); //TODO: this won't always work
					//set connection properties
					debugSession.options.setBoolean("AutoRunToLabelOnRestart", false);
					debugSession.options.setBoolean("ClearBreakpointsOnLoad", true);
					debugSession.options.setBoolean("EnableInstalledBreakpoint",false);
					debugSession.options.setString("VerifyAfterProgramLoad", "Full verification");
					return formatResponse(true, "", "debug session started...");
					
				case CONNECT:
					debugSession.target.connect();
					return formatResponse(true, "", "target connected...");
			
				case DISCONNECT:
					debugSession.target.disconnect();
					return formatResponse(true, "", "target disconnected...");
					
				case RESET:
					debugSession.target.reset();
					return formatResponse(true, "", "target reset...");

				case LOAD:
					debugSession.memory.loadProgram(params[1]); 
					return formatResponse(true, "", "program loaded...");
					
				case RUN:
					debugSession.target.run();
					return formatResponse(true, "", "program ran...");

				case RUN_ASYNCH:
					debugSession.target.runAsynch();
					return formatResponse(true, "", "program running...");
					
				case HALT:
					debugSession.target.halt();
					return formatResponse(true, "", "target halted...");
					
				case IS_HALTED:
					boolean is_halted = debugSession.target.isHalted();
					return formatResponse(true, "" + bool2string(is_halted), "target is halted? - " + bool2string(is_halted) + " ...");
			
				case IS_CONNECTED:
					boolean is_connected = debugSession.target.isConnected();
					return formatResponse(true, "" + bool2string(is_connected),  "target is connected? - " + bool2string(is_connected) + " ...");
					
				case RESTART:
					debugSession.target.restart();
					return formatResponse(true, "", "target restarted...");
					
				case WAIT_HALT:
					debugSession.target.waitForHalt();
					return formatResponse(true, "", "target halted...");
					
				case GET_SYMBOL:
					long symbol_address = debugSession.symbol.getAddress(params[1]);
					return formatResponse(true, "" + symbol_address,  "symbol address? - " + symbol_address + " ...");
					
				case READ:
					int read_page = Integer.parseInt(params[1]);
					long read_addr = Long.parseLong(params[2]);
					int read_length = Integer.parseInt(params[3]);
					int read_size = Integer.parseInt(params[4]);
					boolean read_sign = Boolean.parseBoolean(params[5]);
					long[] read_data = debugSession.memory.readData(read_page, read_addr, read_size, read_length, read_sign); 
					String read_data_string = "";
					for(long read_data_temp: read_data)
					{
						read_data_string += read_data_temp + " ";
					}
					if(read_size == 32){
					if(read_sign){return formatResponse(true, read_data_string, "32-bit memory signed array read...");}
					else {return formatResponse(true, read_data_string, "32-bit memory unsigned array read...");}
					}
					else {
					if(read_sign){return formatResponse(true, read_data_string, "16-bit memory signed array read...");}
					else {return formatResponse(true, read_data_string, "16-bit memory unsigned array read...");}
					}
					
				case WRITE:					
					int write_page = Integer.parseInt(params[1]);
					long write_addr = Long.parseLong(params[2]);
					int write_length = Integer.parseInt(params[3]);
					int write_size = Integer.parseInt(params[4]);
					long[] write_data = new long[write_length];
					for(int i = 0; i < write_length; i++)
					{
						write_data[i] = Long.parseLong(params[i + 5]);
					}
					
					debugSession.memory.writeData(write_page, write_addr, write_data ,write_size);
					if(write_size == 32){return formatResponse(true, "",  "32-bit memory was written...");}
					else {return formatResponse(true, "", "16-bit memory was written...");}
					
				case SET_BREAKPOINT:
					debugSession.breakpoint.add(Long.parseLong(params[1])); 
					return formatResponse(true, "",  "breakpoint set...");
					
				case CLOSE:
					close = true;
					return formatResponse(true, "",  "closing session...");		
					
				case EVALUATE:
					debugSession.expression.evaluate(params[1]);
					return formatResponse(true, "", "expression evaluated...");
					
				case MEMORY_MAP:
					debugSession.memory.map.enable(Boolean.parseBoolean(params[1]));
					if(Boolean.parseBoolean(params[1])) {return formatResponse(true, "", "memory map enabled...");}
					else {return formatResponse(true, "", "memory map disabled...");}
					
				case SINGLE_READ:
					int read_page_single = Integer.parseInt(params[1]);
					long read_addr_single = Long.parseLong(params[2]);
					int read_size_single = Integer.parseInt(params[3]);
					boolean read_sign_single = Boolean.parseBoolean(params[4]);
					long read_data_single = debugSession.memory.readData(read_page_single, read_addr_single, read_size_single, read_sign_single);
					if(read_size_single == 32){
					if(read_sign_single){return formatResponse(true, Long.toString(read_data_single), "32-bit signed double word read...");}
					else{return formatResponse(true, Long.toString(read_data_single), "32-bit unsigned double word read...");}
					}
					else {
					if(read_sign_single){return formatResponse(true, Long.toString(read_data_single), "16-bit signed word read...");}
					else{return formatResponse(true, Long.toString(read_data_single), "16-bit unsigned word read...");}
					}
					
				case READ_REGISTER:
					long register_data = debugSession.memory.readRegister(params[1]);
					return formatResponse(true, Long.toString(register_data), "register read...");
				
				case WRITE_REGISTER:
					debugSession.memory.writeRegister((params[1]), Long.parseLong(params[2]));
					return formatResponse(true, "", "register written...");
			
				case INVALID:
					return formatResponse(false, "", "Invalid command: " + command);
			}
			
		}
		catch (Exception e)
		{
			return formatResponse(false, "","" + e);
		}
	
		return formatResponse(false, "", "Fell through switch statement?");
	}

	private void close()
	{
		tcpServer.close();
		System.exit(0);
	}
	
	private String bool2string(boolean bool)
	{
		if(bool)
		{
			return 1 + "";
		}
		else
		{
			return 0 + "";
		}
	}
	
	public String formatResponse(boolean success, String data, String comments)
	{
		String temp = "";
		if(success)
		{
			temp += "SUCCESS ";
		}
		else
		{
			temp += "ERROR ";
		}
		temp += data;
		temp += " // " + comments;
		return temp;
	}
	
	public static void main(String[] args)
	{
		CommandServer server = new CommandServer();
		server.start();
	}
}